import requests

class SendingClient():
    """
    This class is used to send data to the LRS API
    For the moment the only thing that can be send is statements, wich are converted by this class from data dicts
    """

    def __init__(self,server_addr):
        self.server_addr = server_addr

    def send(self,data,server_path):
        """
        Try to send the data to the LRS and catch the exceptions

        Args : 
            data (object: dict()): the data in the xAPI format
            server_path (str) the server address with the right folder

        Return :
            The API's response
        """
        response = requests.post(self.server_addr+server_path,json = data, timeout=0.25)
        return response


    def change_server_addr(self,server_addr):
        """
        Change the class attribute server_addr

        Args :
            server_addr (str) the server address
        """
        self.server_addr = server_addr


from thonny import get_workbench
from thonny.shell import ShellView

import tkinter as tk
import logging
from datetime import datetime
from copy import deepcopy
import os
import re

from thonnycontrib.thonny_LoggingPlugin.processing import formatData
from thonnycontrib.thonny_LoggingPlugin.configuration import configuration
from thonnycontrib.thonny_LoggingPlugin.configuration.globals import *
from thonnycontrib.thonny_LoggingPlugin.communication.sendingClient import SendingClient
from thonnycontrib.thonny_LoggingPlugin.formats.xAPI_creation import convert_to_xAPI

class EventLogger:
    """
    Main class to generate logs from user's actions
    This class bind the thonny's event generator to the function 'prepare_log'
    When an event is generated :
        Data is extracted
        Additionnal informations can be added
        The data is send to a general formatter in order to have standardised names and informations
        This data in a general format is stored, printed and next, formatted in xAPI format and sended to a server
    """

    def __init__(self):
        """
        Construct an instance of EventLogger, initiates the attributes and makes the binds to get our data
        """
        # Stockage dans la ram des logs pour les enregistrer dans un fichier lors de la fermeture de thonny
        self.events = []
        self.formatted_logs = []

        # Instance de la classe de formatage basique
        self.formatter = formatData.FormatData(self)
        
        #Configs :
        self.sending_client = SendingClient(configuration.get_option("server_address"))
        self.sending_logs = configuration.get_option("send_logs")
        self.log_in_console = configuration.get_option("log_in_console")
        self.store_logs = configuration.get_option("store_logs")
        self.folder = configuration.get_option("local_path")

        #Pour _buffer_errors
        self._stderrBuffer = dict()

        #Pour regroup_tests
        self.regrouped_tests = []


        #Attribut des éléments que l'on veut séléctionner :
        self._inDict = {
            "ShellCommand" : {'sequence','command_text','tags'},
            "ShellInput" : {'sequence','input_text','tags'},
            "TextInsert" : {'sequence','index','text','text_widget','text_widget_context','tags'},

            "Save" : {'sequence','editor','text_widget','filename'},
            "Open" : {'sequence','editor','text_widget','filename'},
            "SaveAs" : {'sequence','editor','text_widget','filename'},

            "l1Tests" : {'sequence','filename','tested_line','expected_result','obtained_result','exception','name','lineno','selected'}
        }

        for sequence in self._inDict:
            self._bind(sequence)
        self._bind("UiCommandDispatched")

        get_workbench().bind("WorkbenchClose", self._on_workbench_close, True)

        #Lance le début de Session du formater ici pour éviter problème de chargement de fichier avant début de session
        self.formatter.begin_session(id(self))

    def _on_workbench_close(self, event):
        """
        write logs in files in the directory specified in the config
        """
        self.formatter.end_session(id(event))
        if self.store_logs:
            try : 
                os.mkdir(self.folder+"/logs")
            except FileExistsError as e :
                pass

            import json
            #Base logs
            with open(self.folder+'/logs/logs'+str(datetime.now())+'.json', encoding="UTF-8", mode="w") as fp:
                json.dump(self.events, fp, indent="    ")
            #Pre formatted logs
            with open(self.folder+'/logs/formatted_logs'+str(datetime.now())+'.json', encoding="UTF-8", mode="w") as fp:
                json.dump(self.formatted_logs,fp,indent="    ")

    def _bind(self,sequence):
        """
        Trigger the prepare_log function when the event type 'sequence' is produced

        Args: 
            sequence (str): the event type

        Returns:
            None
        """
        def handle(event):
            self._prepare_log(sequence,event)
        
        get_workbench().bind(sequence,handle,True)


    def _prepare_log(self,sequence,event):
        """
        extract, process and log the event

        Args:
            sequence (str): the event type
            event (obj:thonny.WorkbenchEvent:) The event
        """
        data = self._extract_interesting_data(event, sequence)

        data["time"] = datetime.now().isoformat()

        data = self._input_processing(data,event)

        self._log_event(data,event)


    def _log_event(self,data,event):
        """
        Store raw data in a buffer and init an event in the formatter

        Args:
            data (obj:'dict'): the raw data
        """
        if data != {}:
            self.events.append(data)
            self.formatter.init_event(data,id(event))

    def _extract_interesting_data(self, event, sequence):
        """
        Extract data from an Thonny's WorkbenchEvent object and select only the informations defined in the in_Dict dictionnary.

        Args :
            event (obj:thonny.WorkbenchEvent) The event to extract data from
            sequence (str) This event type

        Returns:
            (obj:'dict'): the data in the format we want
        """
        attributes = vars(event)
        data = {'tags': () }


        if "text_widget" not in attributes:
            if "editor" in attributes:
                attributes["text_widget"] = attributes["editor"].get_text_widget()

            if "widget" in attributes and isinstance(attributes["widget"], tk.Text):
                attributes["text_widget"] = attributes["widget"]


        if "text_widget" in attributes:
            widget = attributes["text_widget"]
            if isinstance(widget.master.master, ShellView):
                attributes["text_widget_context"] = "shell"


        for elem in self._inDict[sequence]:
            if elem in attributes:
                value = attributes[elem]
                data[elem] = value
                if isinstance(value, (tk.BaseWidget, tk.Tk)):
                    data[elem + "_id"] = id(value)
                    data[elem + "_class"] = value.__class__.__name__

        return data

    def _input_processing(self,data,event):
        """
        Clean and additionnal informations to the data

        Args :
            data (object:'dict') Data to process
            event

        Returns :
            data (object:'dict') Data modified
        """
        # Partie nettoyage
        if 'editor' in data :
            del data['editor']
            del data['editor_class']
        if 'text_widget' in data :
            del data['text_widget']

        #Ajout contenu éditeur
        if data['sequence'] in {'ShellCommand','Open','Save','SaveAs'} :
            data['editorContent'] = get_workbench().get_editor_notebook().get_current_editor_content()

        # Partie traitements
        if data['sequence'] == 'TextInsert' :
            if data["text_widget_class"] == 'ShellText':
                #Gestion cas de sortie trop lourdes pour l'envoi au serveur
                if 'text' in data :
                    if len(data['text']) > 1000 :
                        data['text'] = "TEXT TOO LONG"

                if not 'value' in data['tags'] and not 'stderr' in data['tags'] and not 'stdout' in data['tags']:
                    if 'prompt' in data['tags']:
                        data = self.send_buffer_error()
                        self._log_event(data,event)
                        self.formatter.end_event()
                        return {}
                    return {}
                    
                else :
                    if 'stderr' in data['tags']:
                        data['text'] = self.cut_filename(data['text'])
                        self._buffer_errors(data)
                        return {}
            else : 
                return {}

        if data['sequence'] == 'l1Tests':
            if  'exception' in data or 'obtained_result' in data :
                data['status'] = False
            else :
                data['status'] = True 

            if not 'obtained_result' in data :
                if not 'exception' in data :
                    data['obtained_result'] = data['expected_result']
                else :
                    data['obtained_result'] = data['exception']

        if 'filename' in data :
            data['filename'] = self.cut_filename(data['filename'])
            
        return data


    def _buffer_errors(self,data):
        """
        Store in a buffer the data of user's text edition events and return when the user
        write somewhere else

        Args : 
            data (object:'dict'): Data to process

        """
        buf = deepcopy(self._stderrBuffer)
        if buf == {}:
            self._stderrBuffer = data
        else :
            self._stderrBuffer['text'] = buf['text']+data['text']

    def send_buffer_error(self):
        """
        sends the error buffer
        """
        data = deepcopy(self._stderrBuffer)
        self._stderrBuffer = {}
        return data

    def cut_filename(self,s):
        """
        Cut the filename to keep only the last part

        Args : 
            s (str) the filename

        Return :
            (str) the filename cutted
        
        """
        try :
            return re.search("\/[^\/]*$",s).group()
        except Exception as e :
            return s

    def receive_formatted_logs(self,formatted_log):
        """
        Store and send the formatted logs in parameter to a Server if the user didn't desactivate it
        Args :
            formatted_log (object:'dict') the logs in a basic exportation format
        """
        if formatted_log != {}:
            self.formatted_logs.append(formatted_log)
            if self.log_in_console :
                print(formatted_log)
            if self.sending_logs :
                try :
                        self.send_xAPI_statement(formatted_log)
                except KeyError as e :
                    logging.info(formatted_log,e)
                    return 

    def send_xAPI_statement(self,formatted_log):
        """
        Convert data to the xAPI format, and send it to the LRS

        Args :
            data (object: dict()): dict of the data in a basic format
        """
        xAPI_statement = convert_to_xAPI(formatted_log)
        try :
            self.sending_client.send(xAPI_statement,"/statements/")
        except Exception as e:
            self.sending_logs = False
            logging.warning(" Server can't be reach, restart Thonny to retry\n"+str(e))


from thonny import THONNY_USER_DIR, get_workbench

#Default config
DEFAULT_PATH = THONNY_USER_DIR+"/LoggingPlugin/"
DEFAULT_SERVER = 'http://127.0.0.1:8000'
DEFAULT_STORE = True
DEFAULT_SEND = False
DEFAULT_LOG_IN_CONSOLE = False
#VERSION = "0.1.6"

TERMS_OF_USE = "Acceptez vous que les données utilisateur issues de thonny soient collectées à des fins de recherche\n Plus d'informations dans le menu tools --> LoggingPlugin"
URL_TERMS_OF_USE = "https://www.cristal.univ-lille.fr"

#Dict of options name and default value
OPTIONS = {
    "local_path" : DEFAULT_PATH,
    "server_address" : DEFAULT_SERVER,
    "store_logs" : DEFAULT_STORE,
    "log_in_console" : DEFAULT_LOG_IN_CONSOLE,
    "send_logs" : DEFAULT_SEND,
    "first_run" : True
}

#Get the thonny workbench object
WB = get_workbench()


from thonnycontrib.thonny_LoggingPlugin.configuration.globals import TERMS_OF_USE, URL_TERMS_OF_USE, WB
from thonnycontrib.thonny_LoggingPlugin import mainApp

from thonnycontrib.thonny_LoggingPlugin.configuration import configuration
from thonny.config import *

import tkinter as tk

from thonny import ui_utils
from thonny.ui_utils import CommonDialog


def load_plugin():
    """
    Load the plugin and and a command to configure it in thonny
    """
    configuration.init_options()

    if configuration.get_option("first_run") :
        if display_terms_of_use() :
            configuration.accept_terms_of_uses()
        WB.set_option("loggingPlugin.first_run",False)

    logger = mainApp.EventLogger()
    WB.add_configuration_page("LoggingPlugin", "LoggingPlugin", configuration.plugin_configuration_page, 30)
    
    WB.add_command( command_id="about_logger",
                    menu_name="tools",
                    command_label="Logging Plugin",
                    handler=display_about_plugin)
    return

def display_about_plugin():
    ui_utils.show_dialog(AboutLoggingPlugin(WB))

class AboutLoggingPlugin(CommonDialog):
    def __init__(self, master):
        import webbrowser

        super().__init__(master)

        main_frame = tk.ttk.Frame(self, width = 800, height = 100)
        main_frame.grid(sticky=tk.NSEW, ipadx=50, ipady=100)
        main_frame.rowconfigure(0, weight=1)
        main_frame.columnconfigure(0, weight=1)
        self.title("About Thonny_LoggingPLugin")



        url_font = tk.font.nametofont("TkDefaultFont").copy()
        url_font.configure(underline=1)
        url_label = tk.ttk.Label(
            main_frame, text=URL_TERMS_OF_USE, style="Url.TLabel", cursor="hand2", font=url_font
        )
        url_label.grid()
        url_label.bind("<Button-1>", lambda _: webbrowser.open(URL_TERMS_OF_USE))

def display_terms_of_use():
    return tk.messagebox.askyesno(title="Thonny LoggingPlugin", message=TERMS_OF_USE)


from thonnycontrib.thonny_LoggingPlugin.configuration.globals import OPTIONS,WB

from thonny.config_ui import ConfigurationPage

def init_options():
    for el in OPTIONS :
        if not WB.get_option(el) :
            WB.set_default("loggingPlugin."+el,OPTIONS[el])
    
def get_option(name: str):
        return WB.get_option("loggingPlugin."+name)

def accept_terms_of_uses():
    WB.set_option("loggingPlugin.send_logs",True)

class plugin_configuration_page(ConfigurationPage):
    def __init__(self, master):
        ConfigurationPage.__init__(self, master)
        self.add_checkbox("loggingPlugin.log_in_console","print logs in console ?")
        self.add_checkbox("loggingPlugin.store_logs","store logs in files ?")
        self.add_checkbox("loggingPlugin.send_logs", "Send logs to a server ?")
        #txt
        self.add_entry("loggingPlugin.server_address", row=None, column=0, pady=10, padx=0, columnspan=1,width=100)
        #txt
        self.add_entry("loggingPlugin.local_path", row=None, column=0, pady=10, padx=0, columnspan=1,width=100)


from thonny import get_workbench

from datetime import datetime
import getpass
import hashlib

class FormatData:
    """
        The goal of this class is to format the data in a general way
        wich can be use as a base to convert in other export format

        data is outputed in dict objects as this :
            'run_program' : {'eventID','eventType','timestamp','stdin','stdout','stderr','status','command','userID','sessionID'},
            'run_command' : {'eventID','eventType','timestamp','stdin','stdout','stderr','status','command','userID','sessionID'},
            'openSave' : {'eventID','eventType','timestamp','filename','userID','sessionID'},
            'newfile' : {'eventID','eventType','timestamp','userID','sessionID'},
            'session' : {'eventID','eventType','timestamp','status','userID','sessionID'}
            'l1Tests" : {'eventID','eventType','timestamp','userID','sessionID','status',filename','tested_line','expected_result','obtained_result','exception'}

        In the functions of this class, there is 2 types of events defined :
            The shorts events : Just simples events where the data received in init_event is
                formatted and "ended" in an unique execution of the fonction
            The longs events : Theses are events like commands wich runs the program, where
                we need to get more informations like the inputs made by the user during the execution,
                the results of the executions (stdout,stderr) and all the informations that can occurs
                during the run of a program.
                All theses informations received by init_event when the attribute 'on_progress'
                is True will be processed and stored as it should be, and will be only send when
                the function 'end_event()' will be triggered.
        """


    def __init__(self, logger) -> None:
        self.logger = logger
        self.on_progress = False
        self.current = dict()
        self.userID = hashlib.sha224(bytes(getpass.getuser(),"utf-8")).hexdigest()[:15]
        self.sessionID = id(get_workbench())

    # A lancer lors du prompt '>>>'
    def end_event(self):
        """
        End long event if one is currently active by changing the state of
        the class attribute "on_progress" to False and sending to the EventLogger
        instance the finished formatted log.
        """
        self.on_progress = False
        self.logger.receive_formatted_logs(self.current)
        self.current = dict()

    def init_event(self, data, id):
        """
        Initiate an event by defining if it will be a long event or a short event
        (see the class documentation) and store the data in the class attribute 'current'
        """
        # Si programme ou commande longue lancée :
        if self.on_progress:
            #Ajout des éléments suivant l'execution
            if data['sequence'] == 'TextInsert':
                if 'stdout' in data['tags'] or 'value' in data['tags']:
                    self.current['stdout'] += data['text']
                if 'stderr' in data['tags']:
                    self.current['stderr'] += data['text']
                    self.current['status'] = False

            elif data['sequence'] == 'ShellInput':
                self.current['stdin'] += data['input_text']

        # Cas ou il n'y a pas d'execution en plusieurs temps / seulement des events simples
        else:
            #Informations générales
            self.current['timestamp'] = data['time']
            self.current['eventID'] = id
            self.current['userID'] = self.userID
            self.current['sessionID'] = self.sessionID

            # Cas des runs commandes ou programme
            if data['sequence'] == 'ShellCommand':
                self.format_ShellComand(data)

            # Cas des ouvertures / sauvegardes / nouveau fichiers
            elif data['sequence'] in {'Open', 'Save', 'SaveAS'}:
                self.format_open_save(data)
            

            #Cas non référencés
            
            else :
                self.current['eventType'] = data['sequence']
                for el in data :
                    if el not in self.current and el not in {'sequence','tags'}:
                        self.current[el] = data[el]
                self.end_event()


    def format_ShellComand(self,data):
        """
        Format the current event to be a long event and define if its a Run_program or Run_command
        """
        # On veut avoir les sorties de la commande/du programme
        self.on_progress = True

        # initialisation des champs
        for el in {'stdin', 'stdout', 'stderr'}:
            self.current[el] = ''
        self.current['status'] = True
        self.current['command'] = data['command_text']

        if data['command_text'][:4] == '%Run':
            self.current['eventType'] = 'Run_program'
            self.current['codestate'] = data['editorContent']
        else:
            self.current['eventType'] = 'Run_command'        

    def format_open_save(self,data):
        """
        Make and end an Open / Save / SaveAs event
        """
        if data['sequence'] == 'SaveAs':
            data['sequence'] = 'Save'
        self.current['eventType'] = data['sequence']
        self.current['filename'] = data['filename']
        self.current['codestate'] = data['editorContent']
        self.end_event()


    def begin_session(self, id):
        """
        Create an event "Session_start" and end it immediatly
        Triggered when Thonny is started
        """
        self.current = {
            'eventID': id,
            'eventType': 'Session_start',
            'status': True,
            'timestamp': datetime.now().isoformat(),
            'userID': self.userID,
            'sessionID':  self.sessionID
        }
        self.current['L1Test'] = self.checks_if_plugin_installed('thonnycontrib.l1test')
        self.end_event()

    def end_session(self, id):
        """
        Create an event "Session_end" and end it immediatly
        Triggered when Thonny is closed.
        """
        self.current = {
            'eventID': id,
            'eventType': 'Session_end',
            'status': 'end',
            'timestamp': datetime.now().isoformat(),
            'userID': self.userID,
            'sessionID':  self.sessionID
        }
        self.end_event()

    def isOnProgress(self):
        return self.on_progress

    def checks_if_plugin_installed(self,name):
        import sys
        return name in sys.modules


# This script has as goal to convert data to the format xAPI 

EVENTTYPE_TO_VERB_ID = {
    'Run_program'   : 'https://www.cristal.univ-lille.fr/verbs/Run.Program',
    'Run_command'   : 'https://www.cristal.univ-lille.fr/verbs/Run.Command',
    'Open'          : 'https://www.cristal.univ-lille.fr/verbs/File.Open',
    'Save'          : 'https://www.cristal.univ-lille.fr/verbs/File.Save',
    'Session_start' : 'https://www.cristal.univ-lille.fr/verbs/Session.Start',
    'Session_end'   : 'https://www.cristal.univ-lille.fr/verbs/Session.End',
    'l1Tests'       : 'https://www.cristal.univ-lille.fr/verbs/Run.test'
}

OBJECTS_URI = {
    'file'    : 'https://www.cristal.univ-lille.fr/objects/File',
    'command' : 'https://www.cristal.univ-lille.fr/objects/Command',
    'session' : 'https://www.cristal.univ-lille.fr/objects/Session',
    'test'    : 'https://www.cristal.univ-lille.fr/objects/Test',
    'program' : 'https://www.cristal.univ-lille.fr/objects/Program',
    'plugin'  : 'https://www.cristal.univ-lille.fr/objects/Plugin',
}

def convert_to_xAPI(data):
    """
    Convert the data to xAPI format
    Args :
        data (object: dict()): the data to convert
    Return :
        (object: dict()): the data to xAPI format

    """
    statement = {
        'timestamp' : data['timestamp'],
        'verb' : create_verb(data),
        'actor' : create_actor(data),
        'object' : create_object(data),
        'context' : create_context(data)
    }
    if data['eventType'] in {'Run_program','Run_command','l1Tests'}:
        statement['result'] = create_result(data)

    return statement

def create_actor(data):
    """
    Create an xAPI actor from the data

    Args :
        data (object: dict()): the data to convert
    Return :
        (object: dict()): an actor from the xAPI format
    """
    return {"openid" :'https://www.cristal.univ-lille.fr/users/' + str(data['userID'])}

def create_verb(data):
    """
    Create an xAPI verb from the data

    Args :
        data (object: dict()): the data to convert
    Return :
        (object: dict()): an verb from the xAPI format
    """
    return {'id' : EVENTTYPE_TO_VERB_ID[data['eventType']]}

def create_object(data):
    """
    Create an xAPI object from the data

    Args :
        data (object: dict()): the data to convert
    Return :
        (object: dict()): an object from the xAPI format
    """
    object = dict()
    type = data['eventType']
    if type in ('Open','Save'):
        object['id'] = OBJECTS_URI['file']

        object['extension'] = {
            OBJECTS_URI['file']+'/Filename' : data['filename'],
            OBJECTS_URI['file']+'/CodeState' : data['codestate']
        }

    elif type == 'Run_program':
        object['id'] = OBJECTS_URI['program']
        object['extension'] = {
            OBJECTS_URI['command'] + '/CommandRan' : data['command'],
            OBJECTS_URI['program'] + '/CodeState' : data['codestate']
            }

    elif type == 'Run_command' :
        object['id'] = OBJECTS_URI['command']
        object['extension'] = {OBJECTS_URI['command'] + '/CommandRan' : data['command']}
    
    elif type in {'Session_start','Session_end'} :
        object['id'] = OBJECTS_URI['session']

    elif type == 'l1Tests':
        object['id'] = OBJECTS_URI['test']
        object['extension'] = {
            OBJECTS_URI['file'] + '/Filename' : data['filename'],
            OBJECTS_URI['test'] + '/TestedExpression': data['tested_line'],
            OBJECTS_URI['test'] + '/TestedLine': data['lineno'],
            OBJECTS_URI['test'] + '/TestedFunction': data['name']
        }
    
    else :
        raise Exception("Error : eventType not accepted")
    
    return object

def create_result(data):
    """
    Create an xAPI result from the data

    Args :
        data (object: dict()): the data to convert
    Return :
        (object: dict()): an result from the xAPI format
    """
    result = {
        "success" : data['status']
    }

    if data['eventType'] in {'Run_program','Run_command'}:

        result['extension'] = {
                OBJECTS_URI['command'] +'/stdin'     : data['stdin'],
                OBJECTS_URI['command'] +'/stdout'    : data['stdout'],
                OBJECTS_URI['command'] +'/stderr'    : data['stderr'],
                }

    elif data['eventType'] == 'l1Tests' :

        result['extension'] = {
            OBJECTS_URI['test'] +'/expectedResult' : data['expected_result'],
            OBJECTS_URI['test'] +'/obtainedResult' : data['obtained_result'],
        }

    return result

def create_context(data):
    """
    Create an xAPI context from the data

    Args :
        data (object: dict()): the data to convert
    Return :
        (object: dict()): an context from the xAPI format
    """
    context = {'extension' : { OBJECTS_URI['session'] +'/ID' : str(data['sessionID']) }}
    if data['eventType'] == 'L1Test':
        context['extension'][OBJECTS_URI['plugin']+'/L1Test/SelectedFunction'] = data['selected']
    #pas moyen de le trouver
    if data['eventType'] == 'Session_start' :
        if data['L1Test']  :
            context['extension'][OBJECTS_URI['plugin']] =  OBJECTS_URI['plugin']+'/L1Test'
    return context




import setuptools
import os.path

setupdir = os.path.dirname(__file__)

REQUIREMENTS = ["thonny>=3.3.13"]
VERSION = "0.1.10"

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="thonny-LoggingPlugin",
    version=VERSION,
    author="Corentin",
    author_email="corentin.duvivier.etu@univ-lille.fr",
    description="A plugin that logs and send all the user's actions to an LRS",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.univ-lille.fr/corentin.duvivier.etu/thonny-plugin-journalisation",
    project_urls={
    },
    platforms=["Windows", "macOS", "Linux"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_data={
        "thonnycontrib": ["*.py"],
        "thonnycontrib.thonny_LoggingPlugin": ["*.py"],
        "thonnycontrib.thonny_LoggingPlugin.configuration": ["*.py"],
        "thonnycontrib.thonny_LoggingPlugin.communication": ["*.py"],
        "thonnycontrib.thonny_LoggingPlugin.formats": ["*.py"],
        "thonnycontrib.thonny_LoggingPlugin.processing": ["*.py"],
    },
    packages=[  
        "thonnycontrib",
        "thonnycontrib.thonny_LoggingPlugin",
        "thonnycontrib.thonny_LoggingPlugin.processing",
        "thonnycontrib.thonny_LoggingPlugin.formats",
        "thonnycontrib.thonny_LoggingPlugin.communication",
        "thonnycontrib.thonny_LoggingPlugin.configuration",
    ],
    install_requires=REQUIREMENTS,
    python_requires=">=3.6",
)